@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: mplayer.c,v 1.7 2001/12/19 17:11:40 slouken Exp $
+  $Id: mplayer.c 937 2003-02-12 04:54:31Z slouken $
 
   The Protracker Player Driver
 
@@ -1939,6 +1939,8 @@ void pt_UpdateVoices(int max_volume)
 		tmpvol*=envvol;           /* * max 256 */
 		tmpvol*=pf->volume;       /* * max 128 */
 		tmpvol/=4194304UL;
+               tmpvol*=pf->initvolume;   /* * max 128 */
+               tmpvol/=128;
 
 		/* fade out */
 		if (pf->sngpos>=pf->numpos) tmpvol=0;
